import { Component } from '@angular/core';
import { NavController, LoadingController, ToastController, ActionSheetController, Events } from 'ionic-angular';
import { Slots } from "../../utils/slots";
import { INavigable } from "../../interfaces/navigable-interface";
import { DataProvider } from "../../providers/data-provider";
import { DefaultListable } from "../../defaults/default-listable";
import { SaleOrder } from "../../models/sale.order";
import { BudgetDetailsPage } from "../budget-details/budget-details";

import { OLoader } from "../../components/oloader/oloder";

@Component({
    selector: 'page-budget-list',
    templateUrl: 'budget-list.html'
})
export class BudgetListPage extends DefaultListable<SaleOrder> implements INavigable {

  constructor(
    public navCtrl: NavController,
    public loadingCtrl: LoadingController,
    public toastCtrl: ToastController,
    public actionSheetCtrl: ActionSheetController,
    public events: Events,
    public db: DataProvider
    ) { 
        super(loadingCtrl);
    }
    
    /**
     *
     */
    ionViewDidLoad() {
        this.initialize();

        this.events.subscribe(Slots.ITEM_SAVED, data => {
            this.add(data[0]);
        });
    }

    /**
     *
     */
    ionViewDidLeave() {
        this.events.unsubscribe(Slots.ITEM_SAVED);
    }

    /**
    *
    */
    initialize(): void {
        this.events.publish(Slots.APP_LOADING);

        this.db.getAll(DataProvider.DOCS.SALE_ORDER).then(orders => {
            this.elements = orders.filter(item => {
                // return item.state === "draft" && item.doc_state !== "deleted";
                return true;
            });

            this.events.publish(Slots.APP_LOADED);
        }, e => { 
            console.log(e);

            this.events.publish(Slots.APP_LOADED);

            this.toastCtrl.create({
                message: "No se ha podido cargar los presupuestos",
                duration: 3000
            }).present();
        });
    }

    /**
    *
    */
    goToPage(page: any) {
        this.navCtrl.push(BudgetDetailsPage, this);
    }

    /**
     *
     */
    showOptions(e, item): void {
        this.actionSheetCtrl.create({
            title: "Opciones",
            buttons: [
                {
                    text: "Abrir",
                    icon: "open",
                    handler: () => console.log("open")
                    
                },
                {
                    text: "Eliminar",
                    icon: "trash",
                    role: "destructive",
                    handler: () => console.log("Delete")
                    
                },
                {
                    text: "Cancelar",
                    role: "cancel",
                    handler: () => console.log("Cancel")
                    
                }
            ]
        }).present();
    }
}
